# coding: utf-8

"""
    File Service API v2

    File Service API v2 - Main endpoints (excludes token and tenant management)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.v2.models.auth_integration_type import AuthIntegrationType
from istari_digital_client.v2.models.comment import Comment
from istari_digital_client.v2.models.file import File
from istari_digital_client.v2.models.function_auth_type import FunctionAuthType
from istari_digital_client.v2.models.resource_archive_status import ResourceArchiveStatus
from typing import Optional, Set
from typing_extensions import Self

class AuthIntegration(BaseModel):
    """
    A class representing an Auth Integration.
    """ # noqa: E501
    id: StrictStr
    created: datetime
    file: File
    comments: List[Comment]
    archive_status_history: List[ResourceArchiveStatus]
    created_by_id: StrictStr
    auth_integration_type: AuthIntegrationType
    auth_type: FunctionAuthType
    app_integration_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["id", "created", "file", "comments", "archive_status_history", "created_by_id", "auth_integration_type", "auth_type", "app_integration_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """

        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """

        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """

        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """

        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of file
        if self.file:
            _dict['file'] = self.file.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in comments (list)
        _items = []
        if self.comments:
            for _item_comments in self.comments:
                if _item_comments:
                    _items.append(_item_comments.to_dict())
            _dict['comments'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict['archive_status_history'] = _items
        # set to None if app_integration_id (nullable) is None
        # and model_fields_set contains the field
        if self.app_integration_id is None and "app_integration_id" in self.model_fields_set:
            _dict['app_integration_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AuthIntegration from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "file": File.from_dict(obj["file"]) if obj.get("file") is not None else None,
            "comments": [Comment.from_dict(_item) for _item in obj["comments"]] if obj.get("comments") is not None else None,
            "archive_status_history": [ResourceArchiveStatus.from_dict(_item) for _item in obj["archive_status_history"]] if obj.get("archive_status_history") is not None else None,
            "created_by_id": obj.get("created_by_id"),
            "auth_integration_type": obj.get("auth_integration_type"),
            "auth_type": obj.get("auth_type"),
            "app_integration_id": obj.get("app_integration_id")
        })
        return _obj



from __future__ import annotations

from typing import Any, List, Optional, Protocol, runtime_checkable


@runtime_checkable
class PageLike(Protocol):
    """
    Protocol defining the structure of a paginated response model.

    This protocol is used to verify that a class has all the required fields
    for pagination support before adding Pageable capabilities.
    """

    items: List[Any]
    total: Optional[int]
    page: Optional[int]
    size: Optional[int]
    pages: Optional[int]



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr, StrictBool
from typing import Any, ClassVar, Dict, List, Optional, Set
from typing_extensions import Self

from istari_digital_client.v2.models.token import Token
from istari_digital_client.v2.models.readable import Readable
from istari_digital_client.mixin import ClientHaving


class SnapshotRevisionSearchItem(BaseModel, ClientHaving, Readable):
    """
    SnapshotRevisionSearchItem

    Represents a searchable snapshot revision entry. This includes a token for the
    content and a separate token for metadata properties. The item may be partially
    resolved (e.g. only includes IDs and tokens) and supports methods for reading
    its raw byte content via the associated client.
    """

    file_id: Optional[StrictStr]
    content_token: Token
    properties_token: Token
    created: datetime
    has_artifacts: bool = False
    name: Optional[StrictStr] = None
    extension: Optional[StrictStr] = None
    size: Optional[StrictInt] = None
    description: Optional[StrictStr] = None
    mime: Optional[StrictStr] = None
    version_name: Optional[StrictStr] = None
    external_identifier: Optional[StrictStr] = None
    display_name: Optional[StrictStr] = None
    schema_version: Optional[StrictStr] = None
    created_by_id: Optional[StrictStr] = None
    is_archived: Optional[StrictBool] = False
    resource_id: Optional[StrictStr] = None
    resource_type: Optional[StrictStr] = None
    tenant_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "revision_id",
        "file_id",
        "content_token",
        "properties_token",
        "created",
        "has_artifacts",
        "name",
        "extension",
        "size",
        "description",
        "mime",
        "version_name",
        "external_identifier",
        "display_name",
        "schema_version",
        "created_by_id",
        "is_archived",
        "resource_id",
        "resource_type",
        "tenant_id",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def read_bytes(self) -> bytes:
        """
        Read the byte contents associated with this snapshot.

        This uses the ``content_token`` to retrieve the binary content
        via the client API.

        :raises ValueError: If the client is not set.
        """
        if self._should_use_cache():
            return self._filesystem_caching_read_bytes()

        return self.read_contents()

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of content_token
        if self.content_token:
            _dict["content_token"] = self.content_token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of properties_token
        if self.properties_token:
            _dict["properties_token"] = self.properties_token.to_dict()
        # set to None if file_id (nullable) is None
        # and model_fields_set contains the field
        if self.file_id is None and "file_id" in self.model_fields_set:
            _dict["file_id"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if extension (nullable) is None
        # and model_fields_set contains the field
        if self.extension is None and "extension" in self.model_fields_set:
            _dict["extension"] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict["size"] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if mime (nullable) is None
        # and model_fields_set contains the field
        if self.mime is None and "mime" in self.model_fields_set:
            _dict["mime"] = None

        # set to None if version_name (nullable) is None
        # and model_fields_set contains the field
        if self.version_name is None and "version_name" in self.model_fields_set:
            _dict["version_name"] = None

        # set to None if external_identifier (nullable) is None
        # and model_fields_set contains the field
        if (
            self.external_identifier is None
            and "external_identifier" in self.model_fields_set
        ):
            _dict["external_identifier"] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict["display_name"] = None

        # set to None if schema_version (nullable) is None
        # and model_fields_set contains the field
        if self.schema_version is None and "schema_version" in self.model_fields_set:
            _dict["schema_version"] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict["created_by_id"] = None

        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict["resource_id"] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict["resource_type"] = None

        # set to None if tenant_id (nullable) is None
        # and model_fields_set contains the field
        if self.tenant_id is None and "tenant_id" in self.model_fields_set:
            _dict["tenant_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "revision_id": obj.get("revision_id"),
                "file_id": obj.get("file_id"),
                "content_token": Token.from_dict(obj["content_token"])
                if obj.get("content_token") is not None
                else None,
                "properties_token": Token.from_dict(obj["properties_token"])
                if obj.get("properties_token") is not None
                else None,
                "created": obj.get("created"),
                "has_artifacts": obj.get("has_artifacts"),
                "name": obj.get("name"),
                "extension": obj.get("extension"),
                "size": obj.get("size"),
                "description": obj.get("description"),
                "mime": obj.get("mime"),
                "version_name": obj.get("version_name"),
                "external_identifier": obj.get("external_identifier"),
                "display_name": obj.get("display_name"),
                "schema_version": obj.get("schema_version"),
                "created_by_id": obj.get("created_by_id"),
                "is_archived": obj.get("is_archived")
                if obj.get("is_archived") is not None
                else False,
                "resource_id": obj.get("resource_id"),
                "resource_type": obj.get("resource_type"),
                "tenant_id": obj.get("tenant_id"),
            }
        )
        return _obj



from typing import Annotated

from pydantic import Field

from istari_digital_client.v2.models.google_drive_integration import GoogleDriveIntegration
from istari_digital_client.v2.models.teamwork_cloud_integration import TeamworkCloudIntegration
from istari_digital_client.v2.models.network_drive_integration import NetworkDriveIntegration
from istari_digital_client.v2.models.windchill_integration import WindchillIntegration
from istari_digital_client.v2.models.microsoft365_integration import Microsoft365Integration
from istari_digital_client.v2.models.dassault3d_experience_integration import Dassault3dExperienceIntegration


NewAppIntegration = GoogleDriveIntegration | WindchillIntegration | Microsoft365Integration | Dassault3dExperienceIntegration | TeamworkCloudIntegration | NetworkDriveIntegration



# coding: utf-8

# flake8: noqa
"""
    File Service API v2 - Token and Tenant Management

    File Service API v2 - Token and tenant management endpoints

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from istari_digital_client.storage.models.completed_part_type_def import CompletedPartTypeDef
from istari_digital_client.storage.models.create_multipart_upload_request import CreateMultipartUploadRequest
from istari_digital_client.storage.models.create_multipart_upload_response import CreateMultipartUploadResponse
from istari_digital_client.storage.models.finalize_multipart_upload_action import FinalizeMultipartUploadAction
from istari_digital_client.storage.models.finalize_multipart_upload_request import FinalizeMultipartUploadRequest
from istari_digital_client.storage.models.http_validation_error import HTTPValidationError
from istari_digital_client.storage.models.page_tenant import PageTenant
from istari_digital_client.storage.models.temporary_url import TemporaryURL
from istari_digital_client.storage.models.tenant import Tenant
from istari_digital_client.storage.models.tenant_public_key import TenantPublicKey
from istari_digital_client.storage.models.validation_error import ValidationError
from istari_digital_client.storage.models.validation_error_loc_inner import ValidationErrorLocInner



from __future__ import annotations
import pprint
import re  # noqa: F401
import json
import time
import logging
from datetime import datetime
from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.v2.models.comment import Comment
from istari_digital_client.v2.models.file import File
from istari_digital_client.v2.models.function_version import FunctionVersion
from istari_digital_client.v2.models.job_agent_information import JobAgentInformation
from istari_digital_client.v2.models.job_status import JobStatus
from istari_digital_client.v2.models.job_status_name import JobStatusName
from istari_digital_client.v2.models.resource_archive_status import (
    ResourceArchiveStatus,
)
from istari_digital_client.v2.models.shareable import Shareable
from istari_digital_client.v2.models.file_having import FileHaving
from istari_digital_client.v2.models.archivable import Archivable
from istari_digital_client.mixin import ClientHaving
from istari_digital_client.log_utils import log_method

if TYPE_CHECKING:
    from istari_digital_client.v2.models.model import Model


logger = logging.getLogger(__name__)


class Job(FileHaving, ClientHaving, Shareable, Archivable):
    """
    Represents a job that runs a function on a model and produces a file result.

    A `Job` tracks inputs, execution status, archival history, and related metadata.
    It supports polling for completion, accessing the associated model, and exporting
    structured representations.
    """

    id: StrictStr
    created: datetime
    file: File
    comments: List[Comment]
    archive_status_history: List[ResourceArchiveStatus]
    created_by_id: StrictStr
    function: FunctionVersion
    assigned_agent_id: Optional[StrictStr] = None
    assigned_agent_pool_id: Optional[StrictStr] = None
    agent_information: Optional[JobAgentInformation] = None
    agent_id: Optional[StrictStr] = None
    tenant_id: Optional[StrictStr] = None
    model_id: Optional[StrictStr] = None
    status_history: Optional[List[JobStatus]] = None
    __client_fields__: ClassVar[List[str]] = ["file", "comments"]
    __properties: ClassVar[List[str]] = [
        "id",
        "created",
        "file",
        "comments",
        "archive_status_history",
        "created_by_id",
        "function",
        "assigned_agent_id",
        "agent_id",
        "model_id",
        "status_history",
        "assigned_agent_pool_id",
        "agent_information",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    @log_method
    def archive_status(self) -> ResourceArchiveStatus:
        """
        Get the current archive status of the job.

        This is the last entry in the ``archive_status_history`` list.
        """

        return self.archive_status_history[-1]

    @property
    @log_method
    def model(self) -> Optional["Model"]:
        """
        Retrieve the model associated with this job.

        Returns ``None`` if either ``client`` or ``model_id`` is not set.
        """

        if not self.model_id or not self.client:
            return None
        return self.client.get_model(self.model_id)

    @property
    @log_method
    def status(self) -> JobStatus:
        """
        Get the current status of the job.

        This is the last entry in the ``status_history`` list.

        :raises ValueError: If the ``status_history`` is empty.
        """

        if not self.status_history:
            raise ValueError("Status history is empty")
        return self.status_history[-1]

    @log_method
    def poll_job(self) -> JobStatusName:
        """
        Poll the job until it reaches a terminal state.

        Continuously queries the server for job status every 5 seconds until one of the
        terminal states is reached: ``COMPLETED``, ``FAILED``, or ``CANCELED``.

        :raises ValueError: If the ``client`` is not set.
        """

        if not self.client:
            raise ValueError("Client is not set")
        job = self.client.get_job(self.id)

        logger.info("Begin polling job status for job ID: %s", self.id)
        while job.status.name not in [
            JobStatusName.COMPLETED,
            JobStatusName.FAILED,
            JobStatusName.CANCELED,
        ]:
            time.sleep(5)
            job = self.client.get_job(self.id)
            logger.info("Polling job status: %s", job.status.name)

        logger.info("Finished polling job status for job ID: %s", self.id)
        return job.status.name

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of file
        if self.file:
            _dict["file"] = self.file.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in comments (list)
        _items = []
        if self.comments:
            for _item_comments in self.comments:
                if _item_comments:
                    _items.append(_item_comments.to_dict())
            _dict["comments"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict["archive_status_history"] = _items
        # override the default output from pydantic by calling `to_dict()` of function
        if self.function:
            _dict["function"] = self.function.to_dict()
        # override the default output from pydantic by calling `to_dict()` of agent_information
        if self.agent_information:
            _dict["agent_information"] = self.agent_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in status_history (list)
        _items = []
        if self.status_history:
            for _item_status_history in self.status_history:
                if _item_status_history:
                    _items.append(_item_status_history.to_dict())
            _dict["status_history"] = _items
        # set to None if model_id (nullable) is None
        # and model_fields_set contains the field
        if self.model_id is None and "model_id" in self.model_fields_set:
            _dict["model_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "file": File.from_dict(obj["file"]) if obj.get("file") is not None else None,
            "comments": [Comment.from_dict(_item) for _item in obj["comments"]] if obj.get("comments") is not None else None,
            "archive_status_history": [ResourceArchiveStatus.from_dict(_item) for _item in obj["archive_status_history"]] if obj.get("archive_status_history") is not None else None,
            "created_by_id": obj.get("created_by_id"),
            "function": FunctionVersion.from_dict(obj["function"]) if obj.get("function") is not None else None,
            "assigned_agent_id": obj.get("assigned_agent_id", None),
            "assigned_agent_pool_id": obj.get("assigned_agent_pool_id", None),
            "agent_information": JobAgentInformation.from_dict(obj["agent_information"]) if obj.get("agent_information") is not None else None,
            "agent_id": obj.get("agent_id", None),
            "tenant_id": obj.get("tenant_id"),
            "model_id": obj.get("model_id"),
            "status_history": [JobStatus.from_dict(_item) for _item in obj["status_history"]] if obj.get("status_history") is not None else None
        })
        return _obj



"""
Mixins for istari-digital-client models.

This module contains reusable mixins that provide common functionality
across different API versions.
"""

from istari_digital_client.mixin.client_having import ClientHaving
from istari_digital_client.mixin.pageable import Pageable
from istari_digital_client.protocol import PageLike

__all__ = [
    "ClientHaving",
    "Pageable",
    "PageLike",
]



# coding: utf-8

"""
    File Service API v2 - Token and Tenant Management

    File Service API v2 - Token and tenant management endpoints

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union, TypeAlias

from pydantic import Field, StrictBytes, StrictStr
from typing import Any, List, Optional, Tuple, Union
from typing_extensions import Annotated
from istari_digital_client.storage.models.create_multipart_upload_request import CreateMultipartUploadRequest
from istari_digital_client.storage.models.create_multipart_upload_response import CreateMultipartUploadResponse
from istari_digital_client.storage.models.finalize_multipart_upload_request import FinalizeMultipartUploadRequest
from istari_digital_client.storage.models.page_tenant import PageTenant
from istari_digital_client.storage.models.temporary_url import TemporaryURL
from istari_digital_client.storage.models.tenant_public_key import TenantPublicKey
from istari_digital_client.api_client import ApiClient, RequestSerialized
from istari_digital_client.configuration import Configuration
from istari_digital_client.log_utils import log_method
from istari_digital_client.exceptions import ConflictException

from istari_digital_client.storage.models.completed_part_type_def import CompletedPartTypeDef
from istari_digital_client.storage.models.finalize_multipart_upload_action import FinalizeMultipartUploadAction

from istari_digital_client.v2.models.token import Token
from istari_digital_client.v2.models.token_with_properties import TokenWithProperties
from istari_digital_client.v2.models.properties import Properties
from istari_digital_client.v2.models.source import Source
from istari_digital_client.v2.models.file import File
from istari_digital_client.v2.models.file_revision import FileRevision
from istari_digital_client.v2.models.file_revision_archive_status import FileRevisionArchiveStatus
from istari_digital_client.v2.models.archive_status_name import ArchiveStatusName
from istari_digital_client.v2.models.new_source import NewSource
from istari_digital_client.v2.models.revision_bulk_create_item import RevisionBulkCreateItem

import uuid
from datetime import datetime, timezone
import os
from pathlib import Path

import istari_digital_core

PathLike = Union[str, os.PathLike, Path]
JSON: TypeAlias = dict[str, "JSON"] | list["JSON"] | str | int | float | bool | None


class StorageApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, config: Configuration, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.config = config

    @staticmethod
    def _is_azure_url(url: str) -> bool:
        """
        Check if URL is for Azure Blob Storage based on Rust implementation.

        :param url: URL to check
        :type url: str
        :return: True if URL is for Azure Blob Storage, False otherwise
        :rtype: bool
        """
        return (
            'blob.core.windows.net' in url or
            ('se=' in url and 'sp=' in url and 'sv=' in url)
        )

    @validate_call
    @log_method
    def _upload(
        self,
        temp_url: TemporaryURL,
        file_bytes: bytes,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
    ) -> None:
        """
        Upload bytes to a temporary URL.

        :param temp_url: The temporary URL object containing the upload URL (required)
        :type temp_url: TemporaryURL
        :param file_bytes: The bytes to upload (required)
        :type file_bytes: bytes
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional
        :param _content_type: override content type for the request
        :type _content_type: str, optional
        :param _headers: override headers for the request
        :type _headers: dict, optional
        """

        _param = self.__upload_serialize(
            temp_url=temp_url,
            file_bytes=file_bytes,
            _content_type=_content_type,
            _headers=_headers,
        )

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        # For upload, we don't expect a response body, so we don't need to deserialize
        # The response_deserialize method will handle error checking automatically
        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,  # Success with no response body
            '201': None,  # Created with no response body
        }

        self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    def __upload_serialize(
        self,
        temp_url: TemporaryURL,
        file_bytes: bytes,
        _content_type: str | None,
        _headers: Dict[str, Any] | None,
    ) -> RequestSerialized:
        """
        Serialize the upload request parameters.

        :param temp_url: Temporary URL object containing the upload URL
        :type temp_url: TemporaryURL
        :param file_bytes: Bytes data to upload
        :type file_bytes: bytes
        :param _content_type: Optional content type override
        :type _content_type: str | None
        :param _headers: Optional headers override
        :type _headers: Dict[str, Any] | None
        :return: Serialized request parameters
        :rtype: RequestSerialized
        """

        _collection_formats: Dict[str, str] = {}
        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # Set the request body to the file bytes
        if file_bytes is not None:
            _body_params = file_bytes

        # Set required headers based on the Rust implementation
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _header_params['Content-Type'] = 'application/octet-stream'

        # Always set Content-Length header (required for uploads)
        _header_params['Content-Length'] = str(len(file_bytes)) if file_bytes else '0'

        # Check if this is an Azure URL and add required header
        url = temp_url.url
        if self._is_azure_url(url):
            _header_params['x-ms-blob-type'] = 'BlockBlob'

        # No authentication needed for temporary URLs (they're pre-signed)
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='PUT',  # Based on Rust implementation
            resource_path=temp_url.url,  # Use the full URL from temp_url
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host='',  # Empty host since we're using the full URL
            _request_auth=None
        )

    def _upload_properties(self, properties_token: Token, properties_bytes: bytes, *, upload_url: TemporaryURL | None = None) -> None:
        """
        Upload properties with 409 conflict handling.

        :param properties_token: Token containing SHA and salt for the properties
        :type properties_token: Token
        :param properties_bytes: Serialized properties data to upload
        :type properties_bytes: bytes
        :raises ConflictException: If SHA already exists (caught and ignored)
        """
        try:
            properties_upload_url = self.generate_upload_url(properties_token.sha) if not upload_url else upload_url
            self._upload(
                temp_url=properties_upload_url,
                file_bytes=properties_bytes,
            )
        except ConflictException:
            # SHA already exists, skip upload
            pass

    def _upload_content_single(self, file_bytes: bytes, salt: str | None = None,  *, upload_url: TemporaryURL | None = None) -> Token:
        """
        Upload content using single upload for bytes data.

        :param file_bytes: The bytes to upload
        :type file_bytes: bytes
        :param salt: Optional salt value for creating the token
        :type salt: str | None
        :return: Token containing SHA and salt for the uploaded content
        :rtype: Token
        :raises ConflictException: If SHA already exists (caught and ignored)
        """

        # Create content token with optional salt
        if salt is not None:
            content_token = Token.from_bytes(file_bytes, salt)
        else:
            content_token = Token.from_bytes(file_bytes)

        # Get upload URL and upload content with 409 handling
        try:
            content_upload_url = self.generate_upload_url(content_token.sha) if not upload_url else upload_url
            self._upload(
                temp_url=content_upload_url,
                file_bytes=file_bytes,
            )
        except ConflictException:
            # SHA already exists, skip upload
            pass

        return content_token

    def _upload_content_multipart(self, file_bytes: bytes, salt: str | None = None) -> Token:
        """
        Upload content using multipart upload for large bytes data.

        :param file_bytes: The bytes to upload
        :type file_bytes: bytes
        :param salt: Optional salt value for creating the token
        :type salt: str | None
        :return: Token containing SHA and salt for the uploaded content
        :rtype: Token
        :raises ValueError: If upload fails or parts don't match expected count
        :raises Exception: If multipart upload creation fails
        """

        # Create content token with optional salt
        if salt is not None:
            content_token = Token.from_bytes(file_bytes, salt)
        else:
            content_token = Token.from_bytes(file_bytes)

        chunk_size = self.config.multipart_chunksize or 128 * 1024 * 1024  # type: ignore[operator]
        bytes_len = len(file_bytes)
        # AWS S3 supports up to 10,000 parts.
        chunk_count = (bytes_len // chunk_size) + 1
        if bytes_len % chunk_size == 0:
            # We're right on the boundary.
            chunk_count -= 1

        create_request = CreateMultipartUploadRequest(
            sha=content_token.sha,
            num_parts=chunk_count
        )

        try:
            upload_metadata = self.create_multipart_upload(create_request)
        except Exception as e:
            if "Sha already exists" in str(e):
                return content_token
            else:
                raise

        completed_parts = []
        pos = 0

        for part_num in range(1, len(upload_metadata.upload_urls) + 1):
            upload_url = upload_metadata.upload_urls.get(str(part_num))
            if upload_url is None:
                self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                raise ValueError("invalid state detected, missing upload URL for part")

            end_pos = pos + chunk_size
            if end_pos > bytes_len:
                # This is the last part.
                if part_num != len(upload_metadata.upload_urls):
                    # Should never happen.
                    self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                    raise ValueError("invalid state detected, mismatched number of parts and chunks")
                end_pos = bytes_len

            chunk_data = file_bytes[pos:end_pos]

            try:
                response_data = self._upload_multipart_chunk(upload_url, chunk_data)
            except Exception as e:
                self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                raise

            pos = end_pos

            # Check response status and handle ETag
            is_azure = self._is_azure_url(upload_url.url)

            if response_data.status == 200 and not is_azure:
                # We successfully uploaded the part.
                # We need to convert the header value to a string.
                # The ETag header is always quoted, so we need to strip the quotes.
                etag_header = response_data.getheader("ETag")
                if etag_header:
                    try:
                        e_tag = etag_header.strip('"')
                        completed_parts.append(CompletedPartTypeDef(
                            ETag=e_tag,
                            PartNumber=part_num
                        ))
                    except Exception:
                        self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                        raise ValueError("invalid state detected, ETag header is not valid UTF-8")
                else:
                    self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                    raise ValueError("invalid state detected, ETag header missing")
            elif response_data.status == 201 and is_azure:
                # We successfully staged the block to Azure Storage Blob Service.
                # With Azure Storage Blob Service we only get back a x-ms-request-id header which we don't
                # care about here. All we need to do is register the completed part as when we go to complete
                # the multipart upload we get the list of staged blocks for the SHA/blob from the client and
                # ensure that the number of those matches the number of completed parts we pass to it.
                completed_parts.append(CompletedPartTypeDef(
                    ETag="N/A",
                    PartNumber=part_num
                ))
            else:
                # Something went wrong, cancel the upload and return an error.
                self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
                raise ValueError(f"Upload failed with status {response_data.status}")

        if len(completed_parts) != chunk_count:
            # Should never happen.
            self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
            raise ValueError(
                "invalid state detected, the number of completed parts does not equal the number of chunks")

        try:
            finalize_request = FinalizeMultipartUploadRequest(
                action=FinalizeMultipartUploadAction.COMPLETE,
                sha=content_token.sha,
                upload_id=upload_metadata.upload_id,
                completed_parts=completed_parts
            )
            self.finalize_multipart_upload(finalize_request)
            return content_token
        except Exception as e:
            self._cancel_multipart_upload(content_token.sha, upload_metadata.upload_id)
            raise

    def _cancel_multipart_upload(self, sha: str, upload_id: str) -> None:
        """
        Cancel a multipart upload.

        :param sha: SHA hash of the content being uploaded
        :type sha: str
        :param upload_id: Unique identifier for the multipart upload to cancel
        :type upload_id: str
        """
        cancel_request = FinalizeMultipartUploadRequest(
            action=FinalizeMultipartUploadAction.CANCEL,
            sha=sha,
            upload_id=upload_id,
            completed_parts=None
        )
        self.finalize_multipart_upload(cancel_request)

    def _upload_multipart_chunk(self, upload_url: TemporaryURL, chunk_data: bytes) -> Any:
        """
        Upload a single chunk using the existing _upload method but return response data.

        :param upload_url: Temporary URL for uploading the chunk
        :type upload_url: TemporaryURL
        :param chunk_data: Bytes data for this chunk
        :type chunk_data: bytes
        :return: Raw response data from the upload request
        :rtype: Any
        """
        _param = self.__upload_serialize(
            temp_url=upload_url,
            file_bytes=chunk_data,
            _content_type=None,
            _headers=None,
        )

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=self.config.http_request_timeout_secs,
        )
        response_data.read()

        # Return the raw response data so we can check status and headers
        return response_data

    @validate_call
    @log_method
    def _download(
        self,
        temp_url: TemporaryURL,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
    ) -> bytes:
        """
        Download bytes from a temporary URL.

        :param temp_url: The temporary URL object containing the download URL (required)
        :type temp_url: TemporaryURL
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional
        :param _headers: override headers for the request
        :type _headers: dict, optional
        :return: Downloaded bytes data
        :rtype: bytes
        """

        _param = self.__download_serialize(
            temp_url=temp_url,
            _headers=_headers,
        )

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        # For download, we expect binary content in the response body
        _response_types_map: Dict[str, Optional[str]] = {
            '200': "bytearray",  # Success with binary response body
        }

        response = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

        return bytes(response.data)

    def __download_serialize(
        self,
        temp_url: TemporaryURL,
        _headers: Dict[str, Any] | None,
    ) -> RequestSerialized:
        """
        Serialize the download request parameters.

        :param temp_url: Temporary URL object containing the download URL
        :type temp_url: TemporaryURL
        :param _headers: Optional headers override
        :type _headers: Dict[str, Any] | None
        :return: Serialized request parameters
        :rtype: RequestSerialized
        """

        _collection_formats: Dict[str, str] = {}
        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # No body for download requests
        _body_params = None

        # No authentication needed for temporary URLs (they're pre-signed)
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method='GET',  # Based on Rust implementation
            resource_path=temp_url.url,  # Use the full URL from temp_url
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host='',  # Empty host since we're using the full URL
            _request_auth=None
        )

    @log_method
    def _download_token_data(self, token: Token) -> bytes:
        """
        Download data by SHA and salt, with token validation.

        This method can be used to download any type of data (content or properties)
        that is stored and accessed via a token.

        :param token: Access token for retrieving the data
        :type token: Token
        :return: Downloaded and validated data
        :rtype: bytes
        :raises ValueError: If hash validation fails
        """
        # Get download URL using existing method
        download_url = self.generate_download_url(token.sha)

        # Download data
        data = self._download(temp_url=download_url)

        # Validate data matches expected hash (like Rust implementation)
        if not istari_digital_core.compare_hash(token.sha, token.salt, data):
            raise ValueError("Hash of downloaded data does not match expected hash")

        return data

    @log_method
    def read_contents(self, token: Token) -> bytes:
        """
        Read and return the raw contents of a file associated with a token.

        This method retrieves the file data as a byte stream using the specified token,
        which grants access to the protected resource.

        :param token: Access token for retrieving the file contents.
        :type token: Token
        :return: Raw file content as bytes
        :rtype: bytes
        """

        return self._download_token_data(token)

    @log_method
    def read_properties(self, token: Token) -> Properties:
        """
        Retrieve metadata properties for a file associated with a token.

        This method returns descriptive metadata about the file identified by the provided access token.
        The returned `Properties` object includes non-content attributes such as file name, extension,
        size, MIME type, and optional descriptive metadata.

        Fields include:
        - `name`: Full file name with extension (e.g., "foo.JPG").
        - `stem`: File name without extension (e.g., "foo").
        - `extension`: Lowercased file extension without the dot (e.g., "jpg").
        - `suffix`: File suffix with original casing and a leading dot (e.g., ".JPG").
        - `size`: File size in bytes.
        - `mime_type`: MIME type of the file (e.g., "image/jpeg").
        - `description`: Optional description set on the file.
        - `version_name`: Optional version label.
        - `external_identifier`: Optional external reference ID.
        - `display_name`: Optional user-facing display name.

        :param token: Access token used to retrieve file properties.
        :type token: Token
        :return: Properties object containing file metadata
        :rtype: Properties
        """

        data = self._download_token_data(token)

        return Properties.from_bytes(data)

    @log_method
    def create_revision(
        self,
        file_path: PathLike,
        sources: list[NewSource | str] | None = None,
        display_name: str | None = None,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
    ) -> FileRevision:
        """
        Create a new file revision from a given file path and optional metadata.

        This method uploads a file as a new revision and optionally attaches metadata such as
        sources, display name, description, version name, and an external identifier.

        :param file_path: Filesystem path to the file being uploaded as a revision.
        :type file_path: PathLike
        :param sources: Optional list of sources associated with the revision. Each item may be a
                        NewSource instance or a string source name.
        :type sources: list[NewSource | str] | None
        :param display_name: Optional human-readable name for the revision.
        :type display_name: str | None
        :param description: Optional text describing the revision's contents or purpose.
        :type description: str | None
        :param version_name: Optional user-defined version label for the revision.
        :type version_name: str | None
        :param external_identifier: Optional external system identifier for the revision.
        :type external_identifier: str | None
        :return: Created file revision object
        :rtype: FileRevision
        """

        properties = Properties.from_path(
            path=Path(file_path),
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
            display_name=display_name,
        )

        properties_bytes = properties.to_bytes()

        properties_token = Token.from_bytes(properties_bytes)

        # Upload properties
        self._upload_properties(properties_token, properties_bytes)

        # Read file content
        with open(file_path, 'rb') as f:
            file_bytes = f.read()

        # Upload content - conditional based on file size
        if properties.size <= self.config.multipart_threshold:  # type: ignore[operator]
            # Small file - use regular upload
            content_token = self._upload_content_single(file_bytes)
        else:
            # Large file - use multipart upload
            content_token = self._upload_content_multipart(file_bytes)

        # Create sources list
        source_list = [
            Source(
                revision_id=source if isinstance(source, str) else source.revision_id,
                file_id=None,
                resource_type=None,
                resource_id=None,
                relationship_identifier=None if isinstance(source, str) else source.relationship_identifier,
            )
            for source in (sources or [])
        ]

        # Create FileRevision following the same pattern as from_storage_revision
        file_revision_id = str(uuid.uuid4())

        file_revision_archive_status = FileRevisionArchiveStatus(
            id=str(uuid.uuid4()),
            created=datetime.now(timezone.utc),
            name=ArchiveStatusName.ACTIVE,
            reason="Initial",
            created_by_id=None,
            file_revision_id=file_revision_id,
        )

        return FileRevision(
            id=file_revision_id,
            created=datetime.now(timezone.utc),
            file_id=None,
            content_token=content_token,
            properties_token=properties_token,
            archive_status_history=[file_revision_archive_status],
            name=properties.file_name,
            extension=properties.extension,
            size=properties.size,
            description=properties.description,
            mime=properties.mime,
            version_name=properties.version_name,
            external_identifier=properties.external_identifier,
            display_name=properties.display_name,
            sources=source_list,
            products=None,
            created_by_id=None,
            updated=None,
        )
    @log_method
    def create_revisions_bulk(
        self,
        revision_items: list[RevisionBulkCreateItem]
    ) -> list[FileRevision]:
        """
        Create multiple file revisions in a single bulk operation.

        This method uploads multiple files as new revisions, processing their properties
        and content in batches for improved efficiency. Each revision item can include
        optional metadata such as sources, display name, description, version name,
        and an external identifier.

        Note: This method does not support files larger than the multipart upload threshold.
        For large files, use `create_revision` individually.

        :param revision_items: List of revision creation items, each specifying a file path
                               and optional metadata (sources, description, version_name,
                               external_identifier, display_name).
        :type revision_items: list[RevisionBulkCreateItem]
        :return: List of created file revision objects
        :rtype: list[FileRevision]
        :raises ValueError: If any file exceeds the multipart upload threshold.
        """
        # Deal with properties
        properties_list: list[Properties] = []
        property_information: list[Tuple[Token, bytes]] = []

        for revision in revision_items:
            properties = Properties.from_path(
                path=Path(revision.path),
                description=revision.description,
                version_name=revision.version_name,
                external_identifier=revision.external_identifier,
                display_name=revision.display_name,
            )
            properties_list.append(properties)

            if properties.size > self.config.multipart_threshold:  # type: ignore[operator]
                raise ValueError("Tried to bulk upload file above multipart threshold.")

            properties_bytes = properties.to_bytes()

            properties_token = Token.from_bytes(properties_bytes)

            property_information.append((properties_token, properties_bytes))

        properties_urls = self.generate_upload_urls_bulk(
            [token[0].sha for token in property_information]
        )

        for index, url in enumerate(properties_urls):
            properties_token, properties_bytes = property_information[index]
            # Upload properties
            self._upload_properties(properties_token, properties_bytes, upload_url=url)


        # Deal with content
        content_information: list[Tuple[Token, bytes]] = []

        for revision in revision_items:
            # Read file content
            with open(revision.path, 'rb') as f:
                file_bytes = f.read()

            content_token = Token.from_bytes(file_bytes)

            content_information.append((content_token, file_bytes))

        content_urls = self.generate_upload_urls_bulk(
            [token[0].sha for token in content_information]
        )

        for index, url in enumerate(content_urls):
            content_token, content_bytes = content_information[index]
            # Upload content, passing salt so that we get the same shas as we already calculated
            content_token = self._upload_content_single(content_bytes, salt=content_token.salt, upload_url=url)
            content_information[index] = (content_token, content_bytes)  # overwrite token to make sure it matches

        file_revisions: list[FileRevision] = []

        for index, revision in enumerate(revision_items):
            # Create sources list
            source_list = [
                Source(
                    revision_id=source if isinstance(source, str) else source.revision_id,
                    file_id=None,
                    resource_type=None,
                    resource_id=None,
                    relationship_identifier=None if isinstance(source, str) else source.relationship_identifier,
                )
                for source in (revision.sources or [])
            ]

            # Create FileRevision following the same pattern as from_storage_revision
            file_revision_id = str(uuid.uuid4())

            file_revision_archive_status = FileRevisionArchiveStatus(
                id=str(uuid.uuid4()),
                created=datetime.now(timezone.utc),
                name=ArchiveStatusName.ACTIVE,
                reason="Initial",
                created_by_id=None,
                file_revision_id=file_revision_id,
            )

            properties = properties_list[index]
            content_token = content_information[index][0]
            properties_token = property_information[index][0]

            file_revision =  FileRevision(
                id=file_revision_id,
                created=datetime.now(timezone.utc),
                file_id=None,
                content_token=content_token,
                properties_token=properties_token,
                archive_status_history=[file_revision_archive_status],
                name=properties.file_name,
                extension=properties.extension,
                size=properties.size,
                description=properties.description,
                mime=properties.mime,
                version_name=properties.version_name,
                external_identifier=properties.external_identifier,
                display_name=properties.display_name,
                sources=source_list,
                products=None,
                created_by_id=None,
                updated=None,
            )

            file_revisions.append(file_revision)

        return file_revisions

    @log_method
    def create_secret_revision(
        self,
        file_path: PathLike,
        sources: list[NewSource | str] | None = None,
        display_name: str | None = None,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
    ) -> FileRevision:
        """
        Create a new secret file revision from a given file path and optional metadata.

        This method uploads a file containing secret content as a new revision, and optionally
        associates metadata such as sources, display name, description, version name, and an external identifier.

        :param file_path: Filesystem path to the file being uploaded as a secret revision.
        :type file_path: PathLike
        :param sources: Optional list of sources associated with the revision. Each item may be a
                        NewSource instance or a string source name.
        :type sources: list[NewSource | str] | None
        :param display_name: Optional human-readable name for the revision.
        :type display_name: str | None
        :param description: Optional text describing the revision's contents or purpose.
        :type description: str | None
        :param version_name: Optional user-defined version label for the revision.
        :type version_name: str | None
        :param external_identifier: Optional external system identifier for the revision.
        :type external_identifier: str | None
        :return: Created secret file revision object
        :rtype: FileRevision
        """

        file_path = Path(file_path)

        # Create properties (same as regular revision)
        properties = Properties.from_path(
            path=file_path,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
            display_name=display_name,
        )
        properties_bytes = properties.to_bytes()

        # Read and encrypt content
        with open(file_path, 'rb') as f:
            file_bytes = f.read()

        # Get public key and encrypt content
        tenant_public_key = self.get_tenant_public_key()
        public_key_bytes = tenant_public_key.public_key.encode('utf-8')
        encrypted_content = istari_digital_core.encrypt(public_key_bytes, file_bytes)

        # Create tokens from encrypted content (following Rust pattern)
        content_token = Token.from_bytes(encrypted_content)
        properties_token = Token.from_bytes(properties_bytes, content_token.salt)

        # Upload encrypted content - conditional based on file size
        if properties.size <= self.config.multipart_threshold:  # type: ignore[operator]
            # Small file - use single upload
            content_token = self._upload_content_single(encrypted_content)
        else:
            # Large file - use multipart upload
            content_token = self._upload_content_multipart(encrypted_content)

        # Upload properties
        self._upload_properties(properties_token, properties_bytes)

        # Create sources list (same as regular revision)
        source_list = [
            Source(
                revision_id=source if isinstance(source, str) else source.revision_id,
                file_id=None,
                resource_type=None,
                resource_id=None,
                relationship_identifier=None if isinstance(source, str) else source.relationship_identifier,
            )
            for source in (sources or [])
        ]

        # Create FileRevision (same as regular revision)
        file_revision_id = str(uuid.uuid4())

        file_revision_archive_status = FileRevisionArchiveStatus(
            id=str(uuid.uuid4()),
            created=datetime.now(timezone.utc),
            name=ArchiveStatusName.ACTIVE,
            reason="Initial",
            created_by_id=None,
            file_revision_id=file_revision_id,
        )

        return FileRevision(
            id=file_revision_id,
            created=datetime.now(timezone.utc),
            file_id=None,
            content_token=content_token,
            properties_token=properties_token,
            archive_status_history=[file_revision_archive_status],
            name=properties.file_name,
            extension=properties.extension,
            size=properties.size,
            description=properties.description,
            mime=properties.mime,
            version_name=properties.version_name,
            external_identifier=properties.external_identifier,
            display_name=properties.display_name,
            sources=source_list,
            products=None,
            created_by_id=None,
            updated=None,
        )

    @log_method
    def update_revision_content(
        self,
        file_path: PathLike,
        salt: str,
        sources: list[NewSource | str] | None = None,
        display_name: str | None = None,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
    ) -> FileRevision:
        """
        Update the contents of an existing file revision.

        This method uploads new content from the given file path to update an existing revision.
        A salt value must be provided to locate or identify the revision being updated.
        Optional metadata such as sources, display name, description, version name, and
        external identifier can also be modified.

        :param file_path: Filesystem path to the new file content for the revision.
        :type file_path: PathLike
        :param salt: Unique salt value used to identify or deduplicate the revision update.
        :type salt: str
        :param sources: Optional list of sources associated with the revision. Each item may be a
                        NewSource instance or a string source name.
        :type sources: list[NewSource | str] | None
        :param display_name: Optional human-readable name for the revision.
        :type display_name: str | None
        :param description: Optional text describing the updated revision content.
        :type description: str | None
        :param version_name: Optional user-defined version label for the revision.
        :type version_name: str | None
        :param external_identifier: Optional external system identifier for the revision.
        :type external_identifier: str | None
        :return: Updated file revision object
        :rtype: FileRevision
        """

        file_path = Path(file_path)

        # Create properties
        properties = Properties.from_path(
            path=file_path,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
            display_name=display_name,
        )
        properties_bytes = properties.to_bytes()

        # Create properties token using the provided salt
        properties_token = Token.from_bytes(properties_bytes, salt)

        # Read file content
        with open(file_path, 'rb') as f:
            file_bytes = f.read()

        # Upload content - conditional based on file size, now using existing methods with salt
        if properties.size <= self.config.multipart_threshold:  # type: ignore[operator]
            # Small file - use single upload with existing salt
            content_token = self._upload_content_single(file_bytes, salt)
        else:
            # Large file - use multipart upload with existing salt
            content_token = self._upload_content_multipart(file_bytes, salt)

        # Upload properties
        self._upload_properties(properties_token, properties_bytes)

        # Create sources list
        source_list = [
            Source(
                revision_id=source if isinstance(source, str) else source.revision_id,
                file_id=None,
                resource_type=None,
                resource_id=None,
                relationship_identifier=None if isinstance(source, str) else source.relationship_identifier,
            )
            for source in (sources or [])
        ]

        # Create FileRevision
        file_revision_id = str(uuid.uuid4())

        file_revision_archive_status = FileRevisionArchiveStatus(
            id=str(uuid.uuid4()),
            created=datetime.now(timezone.utc),
            name=ArchiveStatusName.ACTIVE,
            reason="Initial",
            created_by_id=None,
            file_revision_id=file_revision_id,
        )

        return FileRevision(
            id=file_revision_id,
            created=datetime.now(timezone.utc),
            file_id=None,
            content_token=content_token,
            properties_token=properties_token,
            archive_status_history=[file_revision_archive_status],
            name=properties.file_name,
            extension=properties.extension,
            size=properties.size,
            description=properties.description,
            mime=properties.mime,
            version_name=properties.version_name,
            external_identifier=properties.external_identifier,
            display_name=properties.display_name,
            sources=source_list,
            products=None,
            created_by_id=None,
            updated=None,
        )

    @log_method
    def update_revision_properties(
        self,
        file_revision: FileRevision,
        display_name: str | None = None,
        description: str | None = None,
        external_identifier: str | None = None,
        version_name: str | None = None,
    ) -> TokenWithProperties:
        """
        Update metadata properties of an existing file revision.

        This method modifies the metadata associated with a file revision, such as display name,
        description, version name, or external identifier. The revision content itself is not changed.

        :param file_revision: The file revision to update properties for
        :type file_revision: FileRevision
        :param display_name: Optional new human-readable name for the revision
        :type display_name: str | None
        :param description: Optional new text describing the revision
        :type description: str | None
        :param external_identifier: Optional new external system identifier
        :type external_identifier: str | None
        :param version_name: Optional new user-defined version label
        :type version_name: str | None
        :return: Updated file object
        :rtype: File
        :raises ValueError: If file revision doesn't have a file_id
        """

        # Create updated properties using direct field assignment (no path needed)
        updated_properties = Properties(
            file_name=file_revision.name or "",
            size=file_revision.size or 0,
            extension=file_revision.extension or "",
            mime=file_revision.mime,
            description=description if description is not None else file_revision.description,
            version_name=version_name if version_name is not None else file_revision.version_name,
            external_identifier=external_identifier if external_identifier is not None else file_revision.external_identifier,
            display_name=display_name if display_name is not None else file_revision.display_name,
        )

        # Serialize properties to bytes
        properties_bytes = updated_properties.to_bytes()

        # Create properties token using existing salt (key part - maintains identity)
        properties_token = Token.from_bytes(properties_bytes, file_revision.properties_token.salt)

        # Upload updated properties
        self._upload_properties(properties_token, properties_bytes)

        # Create TokenWithProperties from the updated token and properties
        return TokenWithProperties(
            id=str(uuid.uuid4()),
            created=datetime.now(timezone.utc),
            sha=properties_token.sha,
            salt=properties_token.salt,
            name=updated_properties.file_name,
            extension=updated_properties.extension,
            size=updated_properties.size,
            description=updated_properties.description,
            mime=updated_properties.mime,
            version_name=updated_properties.version_name,
            external_identifier=updated_properties.external_identifier,
            display_name=updated_properties.display_name,
        )


    @validate_call
    @log_method
    def create_multipart_upload(
        self,
        create_multipart_upload_request: CreateMultipartUploadRequest,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> CreateMultipartUploadResponse:

        """
        Returns the metadata needed to perform a multipart upload for a given token with the SHA

        :param create_multipart_upload_request: (required)
        :type create_multipart_upload_request: CreateMultipartUploadRequest
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._create_multipart_upload_serialize(
            create_multipart_upload_request=create_multipart_upload_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CreateMultipartUploadResponse",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: CreateMultipartUploadResponse = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _create_multipart_upload_serialize(
        self,
        create_multipart_upload_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if create_multipart_upload_request is not None:
            _body_params = create_multipart_upload_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v2/tokens/multipart-upload',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def create_tenant_public_key(
        self,
        public_key_file: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> TenantPublicKey:

        """
        Creates a tenant public key.

        :param public_key_file: (required)
        :type public_key_file: bytearray
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._create_tenant_public_key_serialize(
            public_key_file=public_key_file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "TenantPublicKey",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: TenantPublicKey = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _create_tenant_public_key_serialize(
        self,
        public_key_file,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if public_key_file is not None:
            _files['public_key_file'] = public_key_file
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v2/tenant/publickey',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def delete_tenant_public_key(
        self,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> None:

        """
        Deletes a tenant public key.

        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._delete_tenant_public_key_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: None = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _delete_tenant_public_key_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/v2/tenant/publickey',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def finalize_multipart_upload(
        self,
        finalize_multipart_upload_request: FinalizeMultipartUploadRequest,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> object:

        """
        Finalize a multipart upload for a given token with the SHA

        :param finalize_multipart_upload_request: (required)
        :type finalize_multipart_upload_request: FinalizeMultipartUploadRequest
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._finalize_multipart_upload_serialize(
            finalize_multipart_upload_request=finalize_multipart_upload_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "object",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: object = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _finalize_multipart_upload_serialize(
        self,
        finalize_multipart_upload_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if finalize_multipart_upload_request is not None:
            _body_params = finalize_multipart_upload_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v2/tokens/multipart-upload',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def generate_download_url(
        self,
        sha: StrictStr,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> TemporaryURL:

        """
        Returns a temporary download url for tokenized resource with SHA value

        :param sha: (required)
        :type sha: str
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._generate_download_url_serialize(
            sha=sha,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TemporaryURL",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: TemporaryURL = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _generate_download_url_serialize(
        self,
        sha,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if sha is not None:
            _path_params['sha'] = sha
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v2/tokens/{sha}/download-url',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def generate_upload_url(
        self,
        sha: StrictStr,
        model_id: Optional[StrictStr] = None,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> TemporaryURL:

        """
        Returns a temporary upload url for token with sha

        :param sha: (required)
        :type sha: str
        :param model_id:
        :type model_id: str
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._generate_upload_url_serialize(
            sha=sha,
            model_id=model_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TemporaryURL",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: TemporaryURL = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _generate_upload_url_serialize(
        self,
        sha,
        model_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if sha is not None:
            _path_params['sha'] = sha
        # process the query parameters
        if model_id is not None:

            _query_params.append(('model_id', model_id))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v2/tokens/{sha}/upload-url',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def generate_upload_urls_bulk(
        self,
        request_body: List[StrictStr],
        model_id: Optional[StrictStr] = None,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> List[TemporaryURL]:

        """
        Returns temporary upload urls for each sha provided

        :param request_body: (required)
        :type request_body: List[str]
        :param model_id:
        :type model_id: str
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._generate_upload_urls_bulk_serialize(
            request_body=request_body,
            model_id=model_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[TemporaryURL]",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: List[TemporaryURL] = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _generate_upload_urls_bulk_serialize(
        self,
        request_body,
        model_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'request_body': '',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if model_id is not None:

            _query_params.append(('model_id', model_id))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if request_body is not None:
            _body_params = request_body


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/v2/tokens/upload-urls_bulk',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def get_tenant_public_key(
        self,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> TenantPublicKey:

        """
        Gets a tenant public key.

        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._get_tenant_public_key_serialize(
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TenantPublicKey",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: TenantPublicKey = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _get_tenant_public_key_serialize(
        self,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v2/tenant/publickey',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def list_tenants(
        self,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="Page number")] = None,
        size: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="Page size")] = None,
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> PageTenant:

        """
        This method lists all tenants (admin only).

        :param page: Page number
        :type page: int
        :param size: Page size
        :type size: int
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._list_tenants_serialize(
            page=page,
            size=size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageTenant",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: PageTenant = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _list_tenants_serialize(
        self,
        page,
        size,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if page is not None:

            _query_params.append(('page', page))

        if size is not None:

            _query_params.append(('size', size))

        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/v2/tenants',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )

    @validate_call
    @log_method
    def update_tenant_public_key(
        self,
        public_key_file: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        http_request_timeout_secs: Annotated[StrictInt | None, Field(ge=0)] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0
    ) -> TenantPublicKey:

        """
        Updates a tenant public key.

        :param public_key_file: (required)
        :type public_key_file: bytearray
        :param http_request_timeout_secs: timeout setting for this request
        :type http_request_timeout_secs: int, optional

        """ # noqa: E501

        _param = self._update_tenant_public_key_serialize(
            public_key_file=public_key_file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TenantPublicKey",
            '422': "HTTPValidationError",
        }

        request_timeout = (
            http_request_timeout_secs if http_request_timeout_secs is not None
            else self.config.http_request_timeout_secs
        )
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=request_timeout,
        )
        response_data.read()

        resp_obj: TenantPublicKey = self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

        return resp_obj

    def _update_tenant_public_key_serialize(
        self,
        public_key_file,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if public_key_file is not None:
            _files['public_key_file'] = public_key_file
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'RequestAuthenticator'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/v2/tenant/publickey',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )



# coding: utf-8

"""
    File Service API v2

    File Service API v2 - Main endpoints (excludes token and tenant management)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class OS(str, Enum):
    """
    OS
    """

    """
    allowed enum values
    """
    WINDOWS_11 = 'Windows 11'
    WINDOWS_10 = 'Windows 10'
    WINDOWS_SERVER_2019 = 'Windows Server 2019'
    WINDOWS_SERVER_2022 = 'Windows Server 2022'
    UBUNTU_24_DOT_04 = 'Ubuntu 24.04'
    UBUNTU_22_DOT_04 = 'Ubuntu 22.04'
    UBUNTU_20_DOT_04 = 'Ubuntu 20.04'
    RHEL_8 = 'RHEL 8'
    RHEL_9 = 'RHEL 9'
    RHEL_10 = 'RHEL 10'
    MACOS_14 = 'macOS 14'
    MACOS_15 = 'macOS 15'
    MACOS_26 = 'macOS 26'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of OS from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
    File Service API v2

    File Service API v2 - Main endpoints (excludes token and tenant management)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List
from typing import Optional, Set
from typing_extensions import Self

class UpstreamRemote(BaseModel):
    """
    UpstreamRemote(id: 'UUID', created: 'Annotated[datetime, AwareDatetime]')
    """ # noqa: E501
    id: StrictStr
    created: datetime
    label: StrictStr
    tenant_label: StrictStr
    tenant_id: StrictStr
    registry_url: StrictStr
    is_archived: StrictBool
    __properties: ClassVar[List[str]] = ["id", "created", "label", "tenant_label", "tenant_id", "registry_url", "is_archived"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """

        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """

        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """

        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """

        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpstreamRemote from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "label": obj.get("label"),
            "tenant_label": obj.get("tenant_label"),
            "tenant_id": obj.get("tenant_id"),
            "registry_url": obj.get("registry_url"),
            "is_archived": obj.get("is_archived")
        })
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.v2.models.control_tag import ControlTag
from istari_digital_client.v2.models.file_archive_status import FileArchiveStatus
from istari_digital_client.v2.models.file_revision import FileRevision
from istari_digital_client.v2.models.file_revision_having import FileRevisionHaving
from istari_digital_client.v2.models.shareable import Shareable
from istari_digital_client.v2.models.archivable import Archivable
from istari_digital_client.mixin import ClientHaving
from istari_digital_client.v2.models.upstream_remote_info import UpstreamRemoteInfo
from istari_digital_client.v2.models.infosec_level import InfosecLevel
from istari_digital_client.log_utils import log_method

if TYPE_CHECKING:
    from istari_digital_client.v2.api.v2_api import Resource


class File(FileRevisionHaving, ClientHaving, Shareable, Archivable):
    """
    Represents a file and its associated metadata, revisions, and archive status.

    Provides convenience methods for retrieving the current revision, linked resource,
    and for updating descriptive properties. Inherits functionality for archiving,
    sharing, and revision access.
    """

    id: StrictStr
    created: datetime
    revisions: List[FileRevision]
    archive_status_history: List[FileArchiveStatus]
    resource_id: Optional[StrictStr] = None
    resource_type: Optional[StrictStr] = None
    created_by_id: Optional[StrictStr] = None
    control_tags: Optional[List[ControlTag]] = None
    updated: Optional[datetime] = None
    upstream_remote_info: Optional[UpstreamRemoteInfo] = None
    infosec_level: Optional[InfosecLevel] = None
    __client_fields__: ClassVar[List[str]] = ["revisions"]
    __properties: ClassVar[List[str]] = [
        "id",
        "created",
        "revisions",
        "archive_status_history",
        "resource_id",
        "resource_type",
        "created_by_id",
        "updated",
        "upstream_remote_info",
        "infosec_level",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    @property
    @log_method
    def revision(self) -> FileRevision:
        """
        Get the latest file revision.

        This is the most recently added item in the ``revisions`` list.
        """

        return self.revisions[-1]

    @property
    @log_method
    def resource(self) -> "Resource":
        """
        Retrieve the resource that this file is attached to.

        Requires that ``client``, ``resource_id``, and ``resource_type`` are all set.

        :raises ValueError: If any of the required values are missing.
        """

        if self.client is None:
            raise ValueError("client is not set")
        if self.resource_id is None:
            raise ValueError("resource_id is not set")
        if self.resource_type is None:
            raise ValueError("resource_type is not set")
        return self.client.get_resource(self.resource_type, self.resource_id)

    @property
    @log_method
    def archive_status(self) -> FileArchiveStatus:
        """
        Get the current archive status of the file.

        This is derived from the last entry in the ``archive_status_history`` list.
        """

        return self.archive_status_history[-1]

    @log_method
    def update_properties(
        self,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        external_identifier: Optional[str] = None,
        version_name: Optional[str] = None,
    ) -> "File":
        """
        Update one or more metadata properties for the latest file revision.

        This method delegates to the SDK client to update properties such as
        ``description``, ``display_name``, ``external_identifier``, and ``version_name``.

        :param description: Optional description for the file.
        :type description: str or None
        :param display_name: Optional human-readable name.
        :type display_name: str or None
        :param external_identifier: Optional external system reference.
        :type external_identifier: str or None
        :param version_name: Optional version label.
        :type version_name: str or None
        :raises ValueError: If the client is not set.
        """

        if self.client is None:
            raise ValueError("client is not set")

        return self.client.update_revision_properties(
            self.revision,
            description=description,
            display_name=display_name,
            external_identifier=external_identifier,
            version_name=version_name,
        )

    @log_method
    def update_description(self, description: str) -> "File":
        """
        Update the file’s description.

        :param description: The new description to apply.
        :type description: str
        """

        return self.update_properties(
            description=description,
        )

    @log_method
    def update_display_name(self, display_name: str) -> "File":
        """
        Update the file’s display name.

        :param display_name: The new display name to apply.
        :type display_name: str
        """

        return self.update_properties(
            display_name=display_name,
        )

    @log_method
    def update_external_identifier(self, external_identifier: str) -> "File":
        """
        Update the file’s external identifier.

        :param external_identifier: The new external identifier to apply.
        :type external_identifier: str
        """

        return self.update_properties(
            external_identifier=external_identifier,
        )

    @log_method
    def update_version_name(self, version_name: str) -> "File":
        """
        Update the file’s version name.

        :param version_name: The new version name to apply.
        :type version_name: str
        """

        return self.update_properties(
            version_name=version_name,
        )

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in revisions (list)
        _items = []
        if self.revisions:
            for _item_revisions in self.revisions:
                if _item_revisions:
                    _items.append(_item_revisions.to_dict())
            _dict["revisions"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict["archive_status_history"] = _items
        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict["resource_id"] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict["resource_type"] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict["created_by_id"] = None

        # set to None if updated (nullable) is None
        # and model_fields_set contains the field
        if self.updated is None and "updated" in self.model_fields_set:
            _dict["updated"] = None

        # set to None if upstream_remote_info (nullable) is None
        # and model_fields_set contains the field
        if (
            self.upstream_remote_info is None
            and "upstream_remote_info" in self.model_fields_set
        ):
            _dict["upstream_remote_info"] = None
        # set to None if infosec_level (nullable) is None
        # and model_fields_set contains the field
        if self.infosec_level is None and "infosec_level" in self.model_fields_set:
            _dict["infosec_level"] = None

        # override the default output from pydantic by calling `to_dict()` of each item in control_tags (list)
        _items = []
        if self.control_tags:
            for _item_control_tags in self.control_tags:
                if _item_control_tags:
                    _items.append(_item_control_tags.to_dict())
            _dict["control_tags"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created": obj.get("created"),
                "revisions": [
                    FileRevision.from_dict(_item) for _item in obj["revisions"]
                ]
                if obj.get("revisions") is not None
                else None,
                "archive_status_history": [
                    FileArchiveStatus.from_dict(_item)
                    for _item in obj["archive_status_history"]
                ]
                if obj.get("archive_status_history") is not None
                else None,
                "resource_id": obj.get("resource_id"),
                "resource_type": obj.get("resource_type"),
                "created_by_id": obj.get("created_by_id"),
                "control_tags": [
                    ControlTag.from_dict(_item) for _item in obj["control_tags"]
                ]
                if obj.get("control_tags") is not None
                else None,
                "updated": obj.get("updated", None)
                if obj.get("updated") is not None
                else None,
                "upstream_remote_info": UpstreamRemoteInfo.from_dict(
                    obj["upstream_remote_info"]
                )
                if obj.get("upstream_remote_info") is not None
                else None,
                "infosec_level": InfosecLevel.from_dict(obj["infosec_level"])
                if obj.get("infosec_level") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
    File Service API v2

    File Service API v2 - Main endpoints (excludes token and tenant management)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class BulkSharingAccessRelation(str, Enum):
    """
    Access relation types for resource bulk sharing operations.  This enum contains only the access relations needed for bulk sharing to avoid circular import issues with the permission module.
    """

    """
    allowed enum values
    """
    VIEWER = 'viewer'
    EDITOR = 'editor'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of BulkSharingAccessRelation from a JSON string"""
        return cls(json.loads(json_str))



"""
Backward compatibility re-export for pagination mixins.

.. deprecated:: 9.0.0
    Importing from istari_digital_client.v2.models.pageable is deprecated.
    Use istari_digital_client.mixin for Pageable,
    or istari_digital_client.protocol for PageLike.
"""

import warnings

from istari_digital_client.mixin import Pageable
from istari_digital_client.protocol import PageLike

warnings.warn(
    "istari_digital_client.v2.models.pageable is deprecated. "
    "Use istari_digital_client.mixin for Pageable "
    "or istari_digital_client.protocol for PageLike.",
    DeprecationWarning,
    stacklevel=2,
)

__all__ = ["Pageable", "PageLike"]



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.v2.models.file import File
from istari_digital_client.v2.models.resource_archive_status import (
    ResourceArchiveStatus,
)
from istari_digital_client.v2.models.file_having import FileHaving
from istari_digital_client.v2.models.archivable import Archivable
from istari_digital_client.mixin import ClientHaving
from istari_digital_client.log_utils import log_method

if TYPE_CHECKING:
    from istari_digital_client.v2.api.v2_api import Resource


class Comment(FileHaving, ClientHaving, Archivable):
    """
    Represents a comment associated with a file and a parent resource.

    A comment includes a file, optional child comments, archive status history, and
    metadata identifying the user and target resource. Inherits client-awareness,
    file-handling, and archive-related functionality.
    """

    id: StrictStr
    created: datetime
    file: File
    comments: List[Comment]
    archive_status_history: List[ResourceArchiveStatus]
    created_by_id: StrictStr
    resource_id: Optional[StrictStr]
    resource_type: Optional[StrictStr] = None
    __client_fields__: ClassVar[List[str]] = ["file", "comments"]
    __properties: ClassVar[List[str]] = [
        "id",
        "created",
        "file",
        "comments",
        "archive_status_history",
        "created_by_id",
        "resource_id",
        "resource_type",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    @log_method
    def archive_status(self) -> ResourceArchiveStatus:
        """
        Get the current archive status of the comment.

        This is computed from the last entry in the ``archive_status_history`` list.
        """

        return self.archive_status_history[-1]

    @property
    @log_method
    def resource(self) -> Optional["Resource"]:
        """
        Retrieve the resource this comment is attached to.

        Requires that ``client``, ``resource_id``, and ``resource_type`` are all set.
        """

        if not self.client or not self.resource_id or not self.resource_type:
            return None
        return self.client.get_resource(self.resource_type, self.resource_id)

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of file
        if self.file:
            _dict["file"] = self.file.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in comments (list)
        _items = []
        if self.comments:
            for _item_comments in self.comments:
                if _item_comments:
                    _items.append(_item_comments.to_dict())
            _dict["comments"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict["archive_status_history"] = _items
        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict["resource_id"] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict["resource_type"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created": obj.get("created"),
                "file": File.from_dict(obj["file"])
                if obj.get("file") is not None
                else None,
                "comments": [Comment.from_dict(_item) for _item in obj["comments"]]
                if obj.get("comments") is not None
                else None,
                "archive_status_history": [
                    ResourceArchiveStatus.from_dict(_item)
                    for _item in obj["archive_status_history"]
                ]
                if obj.get("archive_status_history") is not None
                else None,
                "created_by_id": obj.get("created_by_id"),
                "resource_id": obj.get("resource_id"),
                "resource_type": obj.get("resource_type"),
            }
        )
        return _obj


# TODO: Rewrite to not use raise_errors
Comment.model_rebuild(raise_errors=False)



from __future__ import annotations

from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Generic,
    Iterator,
    List,
    Optional,
    TypeVar,
)

from pydantic import BaseModel, ConfigDict, Field, PrivateAttr
from typing_extensions import Annotated

from istari_digital_client.log_utils import log_method
from istari_digital_client.protocol import PageLike

T = TypeVar("T", bound=BaseModel)

__all__ = ["Pageable", "PageLike"]


class Pageable(BaseModel, Generic[T]):
    """
    A mixin providing iteration capabilities for paginated API responses.

    This mixin adds iter_pages() and iter_items() methods to paginated response classes,
    allowing iteration through all pages and all items across multiple API calls.

    It represents a single page of items returned from a paginated API and includes
    metadata such as total item count, current page number, and total pages.
    """

    items: List[T]
    total: Optional[Annotated[int, Field(strict=True, ge=0)]]
    page: Optional[Annotated[int, Field(strict=True, ge=1)]]
    size: Optional[Annotated[int, Field(strict=True, ge=1)]]
    pages: Optional[Annotated[int, Field(strict=True, ge=0)]] = None
    __client_fields__: ClassVar[List[str]] = ["items"]
    _list_method: Optional[Callable[..., "Pageable[T]"]] = PrivateAttr(default=None)
    _list_method_args: Optional[Dict[str, Any]] = PrivateAttr(default=None)
    __properties: ClassVar[List[str]] = ["items", "total", "page", "size", "pages"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def _validate_pagination_setup(self) -> None:
        """Validate that pagination is properly configured."""
        if self._list_method is None:
            raise ValueError("No list method defined for pagination")
        if self._list_method_args is None:
            raise ValueError("No list method arguments defined for pagination")

    def _get_base_args(self) -> Dict[str, Any]:
        """
        Prepare base arguments for the list method, excluding pagination parameters
        and private attributes.
        """
        return {
            k: v
            for k, v in (self._list_method_args or {}).items()
            if k not in {"page", "size", "self"} and not k.startswith("__")
        }

    @log_method
    def iter_pages(self) -> Iterator["Pageable[T]"]:
        """
        Iterate over all pages, starting with the current page.

        Yields the current page first, then fetches subsequent pages using
        the configured list method.

        :raises ValueError: If either `_list_method` or `_list_method_args` is not set.
        """
        # Early exit for empty results - don't yield empty page
        if self.total == 0:
            return

        # Yield current page first (no API call needed)
        yield self

        # Early exit if this is the only page
        if self.pages and (self.page or 1) >= self.pages:
            return

        self._validate_pagination_setup()
        assert self._list_method is not None  # Validated above, helps mypy

        current_page = (self.page or 1) + 1
        size = self.size or 10
        base_args = self._get_base_args()

        while True:
            page = self._list_method(**base_args, page=current_page, size=size)

            if page.total == 0:
                break

            yield page

            current_page += 1

            if page.pages and current_page > page.pages:
                break

    @log_method
    def iter_items(self) -> Iterator[T]:
        """
        Iterate over all items across all pages, starting with the current page.

        Yields items from the current page first, then fetches subsequent pages
        and yields their items. This method is independent of iter_pages().
        """
        # Yield items from current page first (no API call needed)
        yield from self.items

        # Early exit if this is the only page or no more pages
        if self.total == 0 or (self.pages and (self.page or 1) >= self.pages):
            return

        self._validate_pagination_setup()
        assert self._list_method is not None  # Validated above, helps mypy

        current_page = (self.page or 1) + 1
        size = self.size or 10
        base_args = self._get_base_args()

        while True:
            page = self._list_method(**base_args, page=current_page, size=size)

            if page.total == 0:
                break

            yield from page.items

            current_page += 1

            if page.pages and current_page > page.pages:
                break



# coding: utf-8

"""
    File Service API v2

    File Service API v2 - Main endpoints (excludes token and tenant management)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.v2.models.image_config import ImageConfig
from istari_digital_client.v2.models.json_path_config import JsonPathConfig
from istari_digital_client.v2.models.media_config import MediaConfig
from istari_digital_client.v2.models.model3_d_config import Model3DConfig
from istari_digital_client.v2.models.pdf_config import PDFConfig
from typing import Optional, Set
from typing_extensions import Self

class ViewConfig(BaseModel):
    """
    View configuration for resource links in documents.
    """ # noqa: E501
    var_json: Optional[JsonPathConfig] = Field(default=None, alias="json")
    image: Optional[ImageConfig] = None
    model3d: Optional[Model3DConfig] = None
    pdf: Optional[PDFConfig] = None
    media: Optional[MediaConfig] = None
    __properties: ClassVar[List[str]] = ["json", "image", "model3d", "pdf", "media"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """

        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """

        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """

        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """

        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of var_json
        if self.var_json:
            _dict['json'] = self.var_json.to_dict()
        # override the default output from pydantic by calling `to_dict()` of image
        if self.image:
            _dict['image'] = self.image.to_dict()
        # override the default output from pydantic by calling `to_dict()` of model3d
        if self.model3d:
            _dict['model3d'] = self.model3d.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pdf
        if self.pdf:
            _dict['pdf'] = self.pdf.to_dict()
        # override the default output from pydantic by calling `to_dict()` of media
        if self.media:
            _dict['media'] = self.media.to_dict()
        # set to None if var_json (nullable) is None
        # and model_fields_set contains the field
        if self.var_json is None and "var_json" in self.model_fields_set:
            _dict['json'] = None

        # set to None if image (nullable) is None
        # and model_fields_set contains the field
        if self.image is None and "image" in self.model_fields_set:
            _dict['image'] = None

        # set to None if model3d (nullable) is None
        # and model_fields_set contains the field
        if self.model3d is None and "model3d" in self.model_fields_set:
            _dict['model3d'] = None

        # set to None if pdf (nullable) is None
        # and model_fields_set contains the field
        if self.pdf is None and "pdf" in self.model_fields_set:
            _dict['pdf'] = None

        # set to None if media (nullable) is None
        # and model_fields_set contains the field
        if self.media is None and "media" in self.model_fields_set:
            _dict['media'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ViewConfig from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "json": JsonPathConfig.from_dict(obj["json"]) if obj.get("json") is not None else None,
            "image": ImageConfig.from_dict(obj["image"]) if obj.get("image") is not None else None,
            "model3d": Model3DConfig.from_dict(obj["model3d"]) if obj.get("model3d") is not None else None,
            "pdf": PDFConfig.from_dict(obj["pdf"]) if obj.get("pdf") is not None else None,
            "media": MediaConfig.from_dict(obj["media"]) if obj.get("media") is not None else None
        })
        return _obj



import os
import logging
from pathlib import Path
from dataclasses import field, dataclass
from typing import TypedDict, Literal, Optional

from istari_digital_client.env import env_bool, env_int, env_str, env_cache_root

BearerAuthSetting = TypedDict(
    "BearerAuthSetting",
    {
        "type": Literal["bearer"],
        "in": Literal["header"],
        "key": Literal["Authorization"],
        "value": str,
    },
)

AuthSettings = TypedDict(
    "AuthSettings",
    {
        "RequestAuthenticator": BearerAuthSetting,
    },
    total=False,
)


@dataclass
class Configuration:
    """
    Client configuration for the Istari Digital SDK.

    This class provides runtime configuration options for the SDK, including registry
    connection settings, retry policies, filesystem cache behavior, logging options,
    and multipart upload settings. Values are loaded from environment variables and
    can be overridden at runtime.

    Most configuration values are optional. Defaults are applied via helper functions
    that read from environment variables with appropriate fallbacks.
    """

    registry_url: Optional[str] = field(
        default_factory=env_str("ISTARI_REGISTRY_URL", default=None)
    )
    registry_auth_token: Optional[str] = field(
        default_factory=env_str("ISTARI_REGISTRY_AUTH_TOKEN")
    )
    http_request_timeout_secs: Optional[int] = field(
        default_factory=env_int("ISTARI_CLIENT_HTTP_REQUEST_TIMEOUT_SECS"),
    )
    # === Retry config fields ===
    retry_enabled: Optional[bool] = field(
        default_factory=env_bool("ISTARI_CLIENT_RETRY_ENABLED", default=True)
    )
    retry_max_attempts: Optional[int] = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MAX_ATTEMPTS")
    )
    retry_min_interval_millis: Optional[int] = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MIN_INTERVAL_MILLIS")
    )
    retry_max_interval_millis: Optional[int] = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MAX_INTERVAL_MILLIS")
    )
    # === Filesystem cache config fields ===
    filesystem_cache_enabled: Optional[bool] = field(
        default_factory=env_bool("ISTARI_CLIENT_FILESYSTEM_CACHE_ENABLED", default=True)
    )
    filesystem_cache_root: Path = field(
        default_factory=env_cache_root("ISTARI_CLIENT_FILESYSTEM_CACHE_ROOT")
    )
    filesystem_cache_clean_on_exit: Optional[bool] = field(
        default_factory=env_bool(
            "ISTARI_CLIENT_FILESYSTEM_CACHE_CLEAN_BEFORE_EXIT", default=True
        )
    )
    retry_jitter_enabled: Optional[bool] = field(
        default_factory=env_bool("ISTARI_CLIENT_RETRY_JITTER_ENABLED", default=True)
    )
    # === Multipart upload config fields ===
    multipart_chunksize: Optional[int] = field(
        default_factory=lambda: (
            val if (val := env_int("ISTARI_CLIENT_MULTIPART_CHUNKSIZE")()) is not None
            else 128 * 1024 * 1024
        )
    )

    multipart_threshold: Optional[int] = field(
        default_factory=lambda: (
            val if (val := env_int("ISTARI_CLIENT_MULTIPART_THRESHOLD")()) is not None
            else 2 * 1024 * 1024 * 1024
        )
    )

    # === Logging config fields ===
    log_level: Optional[str] = field(
        default_factory=env_str("ISTARI_CLIENT_LOG_LEVEL", default="INFO")
    )
    log_to_file: Optional[bool] = field(
        default_factory=env_bool("ISTARI_CLIENT_LOG_TO_FILE", default=False)
    )
    log_file_path: Optional[str] = field(
        default_factory=env_str("ISTARI_CLIENT_LOG_FILE_PATH", default=None)
    )

    # === Date and time formats ===
    datetime_format: str = field(init=False, default="%Y-%m-%dT%H:%M:%S.%f%z")
    date_format: str = field(init=False, default="%Y-%m-%d")

    def __post_init__(self) -> None:
        os.environ["ISTARI_REGISTRY_URL"] = self.registry_url or ""
        os.environ["ISTARI_REGISTRY_AUTH_TOKEN"] = self.registry_auth_token or ""

        self._configure_logging()

    def _configure_logging(self) -> None:
        """Configures logging based on the configuration settings."""
        log_level_str = (self.log_level or "INFO").upper()
        log_level = getattr(logging, log_level_str, logging.INFO)

        if not isinstance(log_level, int):
            raise ValueError(f"Invalid log level: {self.log_level}")

        logger = logging.getLogger("istari_digital_client")
        logger.setLevel(log_level)

        formatter = logging.Formatter(
            "%(asctime)s -  %(name)s:%(funcName)s:%(lineno)d - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

        handlers: list[logging.Handler] = []

        if self.log_to_file:
            if not self.log_file_path:
                raise ConfigurationError(
                    "ISTARI_CLIENT_LOG_FILE_PATH must be set when ISTARI_CLIENT_LOG_TO_FILE=true"
                )

            log_dir = Path(self.log_file_path).parent
            if not log_dir.exists():
                raise ConfigurationError(
                    f"Directory does not exist for log file: {log_dir}"
                )

            file_handler = logging.FileHandler(self.log_file_path)
            file_handler.setFormatter(formatter)
            handlers.append(file_handler)

        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(formatter)
        handlers.append(stream_handler)

        for handler in handlers:
            logger.addHandler(handler)

        logger.propagate = False

    def auth_settings(self) -> AuthSettings:
        """Gets Auth Settings dict for api client.

        :return: The Auth Settings information dict.
        """
        auth: AuthSettings = {}
        if self.registry_auth_token is not None:
            auth["RequestAuthenticator"] = {
                "type": "bearer",
                "in": "header",
                "key": "Authorization",
                "value": "Bearer " + self.registry_auth_token,
            }
        return auth


class ConfigurationError(ValueError):
    pass



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.mixin import ClientHaving
from istari_digital_client.log_utils import log_method

if TYPE_CHECKING:
    from istari_digital_client.v2.models.file import File
    from istari_digital_client.v2.api.v2_api import Resource
    from istari_digital_client.v2.models.file_revision import FileRevision


class Source(BaseModel, ClientHaving):
    """
    Represents an upstream source that contributed to the creation of another resource.

    A Source links a file revision to a target resource such as a model, snapshot, or job.
    It captures the lineage and relationship metadata describing how a particular file
    revision was used during processing or composition.

    This class supports resolution of the original file, its revision, and the associated
    resource, provided that the client is initialized.
    """

    revision_id: StrictStr
    file_id: Optional[StrictStr] = None
    resource_type: Optional[StrictStr] = None
    resource_id: Optional[StrictStr] = None
    relationship_identifier: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "revision_id",
        "file_id",
        "resource_type",
        "resource_id",
        "relationship_identifier",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    @log_method
    def file(self) -> Optional["File"]:
        """
        Retrieve the file associated with this source.

        This requires that ``file_id`` is set and that the client is available.
        If either is missing, this returns ``None``.
        """
        if self.file_id is None or self.client is None:
            return None
        return self.client.get_file(self.file_id)

    @property
    @log_method
    def resource(self) -> Optional["Resource"]:
        """
        Retrieve the resource this source was used to support.

        This requires that ``resource_type`` and ``resource_id`` are set,
        and that the client is available. If any of these are missing, this returns ``None``.
        """
        if (
            self.resource_type is None
            or self.resource_id is None
            or self.client is None
        ):
            return None
        return self.client.get_resource(self.resource_type, self.resource_id)

    @property
    @log_method
    def revision(self) -> Optional["FileRevision"]:
        """
        Retrieve the file revision that represents the source content.

        This requires that ``revision_id`` is set and that the client is available.
        If either is missing, this returns ``None``.
        """
        if self.revision_id is None or self.client is None:
            return None
        return self.client.get_revision(self.revision_id)

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if file_id (nullable) is None
        # and model_fields_set contains the field
        if self.file_id is None and "file_id" in self.model_fields_set:
            _dict["file_id"] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict["resource_type"] = None

        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict["resource_id"] = None

        # set to None if relationship_identifier (nullable) is None
        # and model_fields_set contains the field
        if (
            self.relationship_identifier is None
            and "relationship_identifier" in self.model_fields_set
        ):
            _dict["relationship_identifier"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "revision_id": obj.get("revision_id"),
                "file_id": obj.get("file_id"),
                "resource_type": obj.get("resource_type"),
                "resource_id": obj.get("resource_id"),
                "relationship_identifier": obj.get("relationship_identifier"),
            }
        )
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set
from typing_extensions import Self

from istari_digital_client.v2.models.agent_display_name import AgentDisplayName
from istari_digital_client.v2.models.agent_information import AgentInformation
from istari_digital_client.v2.models.agent_modules import AgentModules
from istari_digital_client.v2.models.agent_status import AgentStatus
from istari_digital_client.log_utils import log_method


class Agent(BaseModel):
    """
    Agent
    """  # noqa: E501

    id: StrictStr
    created: datetime
    display_name_history: Optional[List[AgentDisplayName]] = None
    information_history: Optional[List[AgentInformation]] = None
    status_history: Optional[List[AgentStatus]] = None
    modules_history: Optional[List[AgentModules]] = None
    archive_status: StrictStr
    created_by_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "id",
        "created",
        "information_history",
        "status_history",
        "modules_history",
        "archive_status",
        "created_by_id",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    @log_method
    def display_name(self) -> Optional[AgentDisplayName]:
        """Returns the latest display name record"""
        return self.display_name_history[-1] if self.display_name_history else None

    @property
    @log_method
    def information(self) -> Optional[AgentInformation]:
        """Returns the latest information record"""
        return self.information_history[-1] if self.information_history else None

    @property
    @log_method
    def agent_version(self) -> Optional[str]:
        """Returns the latest agent version"""
        return self.information.agent_version if self.information else None

    @property
    @log_method
    def host_os(self) -> Optional[str]:
        """Returns the latest host OS"""
        return self.information.host_os if self.information else None

    @property
    @log_method
    def status(self) -> Optional[AgentStatus]:
        """Returns the latest status record"""
        return self.status_history[-1] if self.status_history else None

    @property
    @log_method
    def modules(self) -> Optional[AgentModules]:
        """Returns the latest modules record"""
        return self.modules_history[-1] if self.modules_history else None

    def to_str(self) -> str:
        """
        Return the string representation of the model using field aliases.

        This method serializes the model to a JSON-formatted string, respecting any defined aliases
        for fields instead of their original attribute names.
        """

        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """
        Return the JSON string representation of the model using field aliases.

        This method serializes the model into a JSON-formatted string, using aliases for field names
        where defined instead of their raw attribute names.
        """

        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """
        Create an instance of the model from a JSON string.

        This method deserializes the given JSON string into a model instance. It expects the
        string to match the model's schema, using field aliases where applicable.

        :param json_str: JSON string representing the model.
        :type json_str: str
        """

        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """
        Return the dictionary representation of the model using field aliases.

        This method differs from calling `self.model_dump(by_alias=True)` in the following way:

        - Fields with a value of `None` are included in the output only if they are nullable and
          were explicitly set during model initialization. All other `None` fields are omitted.
        """

        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in display_name_history (list)
        _items = []
        if self.display_name_history:
            for _item_display_name_history in self.display_name_history:
                if _item_display_name_history:
                    _items.append(_item_display_name_history.to_dict())
            _dict["display_name_history"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in information_history (list)
        _items = []
        if self.information_history:
            for _item_information_history in self.information_history:
                if _item_information_history:
                    _items.append(_item_information_history.to_dict())
            _dict["information_history"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in status_history (list)
        _items = []
        if self.status_history:
            for _item_status_history in self.status_history:
                if _item_status_history:
                    _items.append(_item_status_history.to_dict())
            _dict["status_history"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in modules_history (list)
        _items = []
        if self.modules_history:
            for _item_modules_history in self.modules_history:
                if _item_modules_history:
                    _items.append(_item_modules_history.to_dict())
            _dict["modules_history"] = _items
        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict["created_by_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """
        Create an instance of the model from a dictionary.

        This method deserializes a dictionary into a model instance. The input should use
        field aliases where applicable.

        :param obj: Dictionary representing the model.
        :type obj: Optional[Dict[str, Any]]
        """

        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created": obj.get("created"),
                "display_name_history": [
                    AgentDisplayName.from_dict(_item)
                    for _item in obj["display_name_history"]
                ]
                if obj.get("display_name_history") is not None
                else None,
                "information_history": [
                    AgentInformation.from_dict(_item)
                    for _item in obj["information_history"]
                ]
                if obj.get("information_history") is not None
                else None,
                "status_history": [
                    AgentStatus.from_dict(_item) for _item in obj["status_history"]
                ]
                if obj.get("status_history") is not None
                else None,
                "modules_history": [
                    AgentModules.from_dict(_item) for _item in obj["modules_history"]
                ]
                if obj.get("modules_history") is not None
                else None,
                "archive_status": obj.get("archive_status"),
                "created_by_id": obj.get("created_by_id"),
            }
        )
        return _obj
